@@ -16,7 +16,9 @@ fame <- function() {
   }
 
 # require("DBI") for this
-setClass("TSfameConnection", contains=c("DBIConnection", "conType", "TSdb"))
+setClass("TSfameConnection", contains=c("DBIConnection", "conType", "TSdb"),
+   representation(current="character"))
+   # current is only used with vintages (faking info the db should have)
    #user / password / host  for future consideration
    # different for read or write accessMode = "character"
    
@@ -32,28 +34,54 @@ setMethod("dbUnloadDriver", signature(drv="fameDriver"),
 
 setMethod("TSconnect",   signature(drv="fameDriver", dbname="character"),
   definition= function(drv, dbname, 
-              accessMode = if(file.exists(dbname)) "shared" else "create", ...){
+              accessMode = if(file.exists(dbname)) "shared" else "create", 
+	      current=NA, ...){
+   #specifying current is a way to fake vintage info the db should have
    #It might be possible to leave the Fame db open, but getfame needs it closed.
    if (is.null(dbname)) stop("dbname must be specified")
+   
    #ensure the db name ends in .db, otherwise fame adds this and then con fails
-   db <- sub('$', '.db',sub('.db$', '', dbname))
+   # if dbname is a vector (for vintages) it should have names so that
+   #  dbname[vintage] can be used to subset it in TSget.
+   nm <- names(dbname)
+   dbname <- sub('$', '.db',sub('.db$', '', dbname))
+   if(is.null(nm) & (1 < length(dbname))) {
+      nm <- as.character(seq(length(dbname)))
+      warning("vintage names generated as sequence: ", nm)
+      }
+   names(dbname) <- nm
+   
    if(!fameRunning()) fameStart(workingDB = FALSE)
-   Id <- try(fameDbOpen(dbname, accessMode = accessMode))
-   if(inherits(Id, "try-error") ) stop("Could not establish TSfameConnection to ", dbname)
-   fameDbClose(Id) # this Id is not saved
+   for (i in seq(length(dbname))){
+      Id <- try(fameDbOpen(dbname[i], accessMode = accessMode))
+      if(inherits(Id, "try-error") )
+         stop("Could not establish TSfameConnection to ", dbname[i])
+      fameDbClose(Id) # this Id is not saved
+      }
    new("TSfameConnection", drv="fame",
-          dbname=dbname, hasVintages=FALSE, hasPanels=FALSE) 
+          dbname=dbname, hasVintages= (1 < length(dbname)), hasPanels=FALSE,
+	  current=as.character(current)) 
    } )
 
 
 setMethod("TSdates",  
    signature(serIDs="character", con="TSfameConnection", vintage="ANY", panel="ANY"),
-   definition= function(serIDs, con, vintage=NULL, panel=NULL, ... )  
-{  # Indicate  dates for which data is available.
+   definition= function(serIDs, con, vintage=getOption("TSvintage"),
+    panel=NULL, ... ) { 
+   # Indicate  dates for which data is available. Only one vintage can be specified.
    # This requires retrieving series individually so they are not truncated.
+   if(con@hasVintages){
+     if(is.null(vintage)) vintage <- "current"
+     if(1 != length(vintage))
+        stop("only one vintage must be specified for TSdates.")
+     if("current" == vintage) vintage <- con@current
+     if(is.na(vintage))
+ 	 stop("connection does not have a specified current vintage.")
+     } 
+
    r <- av <- st <- en <- tb <- NULL
    for (i in 1:length(serIDs))
-     {r <- try(TSget( serIDs[i], con=con), silent=TRUE) 
+     {r <- try(TSget( serIDs[i], con=con, vintage=vintage), silent=TRUE) 
       if(inherits(r, "try-error") ) {
         av <- c(av, FALSE)
 	st <- append(st, list(NA))
@@ -80,15 +108,37 @@ setMethod("TSdates",
 setMethod("TSget",     signature(serIDs="character", con="TSfameConnection"),
    definition= function(serIDs, con, TSrepresentation=getOption("TSrepresentation"),
        tf=NULL, start=tfstart(tf), end=tfend(tf),
-       names=serIDs, TSdescription=FALSE, TSdoc=FALSE, TSlabel=FALSE, ...)
+       names=NULL, TSdescription=FALSE, TSdoc=FALSE, TSlabel=FALSE,
+       vintage=getOption("TSvintage"), ...)
 { # ... arguments unused
   if (is.null(TSrepresentation)) TSrepresentation <- "default"
+  if ( 1 < sum(c(length(serIDs), length(vintage)) > 1))
+   stop("Only one of serIDs or vintage can have length greater than 1.")
+
+  if(con@hasVintages){
+    if(is.null(names)) names <- 
+         if ( length(vintage) > 1 ) vintage  else  serIDs 
+    if(is.null(vintage)) vintage <- "current"
+    vintage["current" == vintage] <- con@current
+    if(any(is.na(vintage)))
+        stop("connection does not have a specified current vintage.")
+    # if vintage is a vector then serIDs needs to be expanded, 
+    #  otherwise dbname needs to be expanded.
+    dbname <- con@dbname[vintage]
+    if ( 1 == length(vintage)) dbname  <- rep(dbname, length(serIDs))
+    else  serIDs <- rep(serIDs, length(vintage))
+    } 
+  else {
+    if(is.null(names)) names <- serIDs 
+    dbname <- rep(con@dbname, length(serIDs))   
+  }
+  
   mat <- desc <- doc <- label <-  rp <- NULL
   for (i in seq(length(serIDs))) {
-    r <- getfame(serIDs[i], con@dbname, save = FALSE, envir = parent.frame(),
+    r <- getfame(serIDs[i], dbname[i], save = FALSE, envir = parent.frame(),
              start = NULL, end = NULL, getDoc = FALSE)
     if(0==length(r))
-       stop("Fame retrieval failed. Series may not exist on ",con@dbname,".")
+       stop("Fame retrieval failed. Series may not exist on ",dbname[i],".")
     # r is class tis
 #    r <-  if((TSrepresentation=="default" | TSrepresentation=="ts")
 #             && frequency(r) %in% c(1,4,12,2)) as.ts(r[[1]]) else as.zoo(r[[1]])
@@ -118,12 +168,12 @@ setMethod("TSget",     signature(serIDs="character", con="TSfameConnection"),
       mat <- changeTSrepresentation(mat, TSrepresentation)
       }
 
-  seriesNames(mat) <- if(!is.null(names)) names else serIDs 
+  seriesNames(mat) <- names 
 
-  TSmeta(mat) <- new("TSmeta", serIDs=serIDs, dbname=con@dbname, 
+  TSmeta(mat) <- new("TSmeta", serIDs=serIDs, dbname=dbname, 
       hasVintages=con@hasVintages, hasPanels=con@hasPanels,
       conType=class(con), DateStamp=Sys.time(), 
-      TSdescription=if(TSdescription) paste(desc, " from ", con@dbname, 
+      TSdescription=if(TSdescription) paste(desc, " from ", dbname, 
             "retrieved ", Sys.time()) else as(NA, "character"), 
       TSdoc=if(TSdoc) doc else as(NA, "character"),
       TSlabel=if(TSlabel) label else as(NA, "character"))
@@ -137,6 +187,8 @@ setMethod("TSput",     signature(x="ANY", serIDs="character", con="TSfameConnect
        warn=TRUE, ...) 
  {
   if (!is.null(TSlabel.)) warning("TSlabel is not supported in Fame.")
+  if (con@hasVintages)
+    stop("TSput does not support vintages. Open the con to a single dbname.")
   ids <-  serIDs 
   x <- as.matrix(as.tis(x)) # clobbers seriesNames(x)
   #ids <- gsub(" ", "", serIDs ) # remove spaces in id
@@ -172,11 +224,11 @@ setMethod("TSput",     signature(x="ANY", serIDs="character", con="TSfameConnect
 
 setMethod("TSdescription",   signature(x="character", con="TSfameConnection"),
    definition= function(x, con=getOption("TSconnection"), ...)
-     fameWhats(con@dbname, x, getDoc = TRUE)$des )
+     fameWhats(con@dbname[1], x, getDoc = TRUE)$des )
 
 setMethod("TSdoc",   signature(x="character", con="TSfameConnection"),
    definition= function(x, con=getOption("TSconnection"), ...)
-     fameWhats(con@dbname, x, getDoc = TRUE)$doc )
+     fameWhats(con@dbname[1], x, getDoc = TRUE)$doc )
 
 #TSlabel gets used for new("Meta", so issuing a warning is not a good idea here.
 setMethod("TSlabel",   signature(x="character", con="TSfameConnection"),
@@ -187,6 +239,8 @@ setMethod("TSdelete",
    signature(serIDs="character", con="TSfameConnection", vintage="ANY", panel="ANY"),
    definition= function(serIDs, con=getOption("TSconnection"),  
             vintage=getOption("TSvintage"), panel=getOption("TSpanel"), ...){
+    if (con@hasVintages)
+       stop("TSdelete does not support vintages. Open the con to a single dbname.")
     ok <- TRUE
     for (i in seq(length(serIDs))) 
       ok <- ok & 0 == fameDeleteObject(con@dbname, serIDs[i]) 
@@ -201,6 +255,8 @@ setMethod("TSexists",
  signature(serIDs="character", con="TSfameConnection", vintage="ANY", panel="ANY"),
  definition= function(serIDs, con=getOption("TSconnection"), 
                       vintage=NULL, panel=NULL, ...){
+   if (con@hasVintages)
+       stop("TSexists does not support vintages. Open the con to a single dbname.")
    op <- options(warn=-1)
    on.exit(options(op))
    ok <- fameWhats(con@dbname, serIDs, getDoc = FALSE)
